/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2010 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.glassfish.dasrecovery;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.*;
import org.jvnet.hk2.annotations.*;
import org.jvnet.hk2.component.*;
import org.glassfish.api.Param;
import org.glassfish.api.admin.*;
import static com.sun.enterprise.admin.cli.CLIConstants.*;
import com.sun.enterprise.universal.i18n.LocalStringsImpl;
import com.sun.enterprise.util.StringUtils;
import com.sun.enterprise.util.net.NetUtils;
import com.sun.enterprise.admin.cli.cluster.LocalInstanceCommand;
import com.sun.enterprise.util.SystemPropertyConstants;

/**
 *  This is a local command that updates the das.properties
 *  under nodes/<node name>/agent/config/
 */
@Service(name = "update-admin-server-local-coordinates")
@Scoped(PerLookup.class)
public class UpdateLocalInstanceDasPropsCommand extends LocalInstanceCommand {
    
    @Param(name = "adminhost", optional = false)
    private String DASHost;  //das.properties das.host

    @Param(name = "adminport", optional = false)
    private String DASPort; //das.properties das.port

    // We are overriding LocalInstanceCommand's definition of node.  In
    // this usage case it is an operand.
    @Param(name = "node", optional = false, primary=true)
    private String node;

    String DASProtocol;
    boolean dasIsSecure;

    private File agentConfigDir;
    private File dasPropsFile;
    private Properties dasProperties;

    private static final LocalStringsImpl strings =
            new LocalStringsImpl(UpdateLocalInstanceDasPropsCommand.class);


    /**
     */
    @Override
    protected void validate()
            throws CommandException {


        /* node dir - parent directory of all node(s) */
        String nodeDirRootPath = null;
        if(ok(nodeDir))
            nodeDirRootPath = nodeDir;
        else // node dir = <install-root>/nodes
            nodeDirRootPath = getNodeDirRootDefault();

        nodeDirChild = new File(nodeDirRootPath, node);

        if (!nodeDirChild.isDirectory()) {
            throw new CommandException(
                strings.get("UpdateDasProps.nodeNotExist", 
                            nodeDirChild.toString()));
        }

        //super.validate();

        String agentPath = "agent" + File.separator + "config";
        agentConfigDir = new File(nodeDirChild, agentPath);

        dasPropsFile = new File(agentConfigDir, "das.properties");

        if (!dasPropsFile.isFile()) {
            throw new CommandException(
                strings.get("UpdateDasProps.propsNotExist",
                             dasPropsFile.toString()));
        }

        if (!NetUtils.isPortStringValid(DASPort)) {
            throw new CommandException(
                strings.get("UpdateDasProps.invalidDasPort", DASPort));
        }

        dasIsSecure = programOpts.isSecure();
        DASProtocol = "http";
    }

    /**
     */
    @Override
    protected int executeCommand()
            throws CommandException {
        updateDasProperties();
        return SUCCESS;
    }

    private void updateDasProperties() throws CommandException {
        try {
            dasPropsFile.createNewFile();
            dasProperties = new Properties();
            dasProperties.setProperty(K_DAS_HOST, DASHost);
            dasProperties.setProperty(K_DAS_PORT, String.valueOf(DASPort));
            dasProperties.setProperty(K_DAS_IS_SECURE, String.valueOf(dasIsSecure));
            dasProperties.setProperty(K_DAS_PROTOCOL, DASProtocol);

            FileOutputStream fos = new FileOutputStream(dasPropsFile);
            dasProperties.store(fos, strings.get("UpdateDasProps.dasPropertyComment"));
            logger.finer("das.properties values set is " + K_DAS_HOST + "=" + DASHost +
                                                             "," + K_DAS_PORT + "=" + DASPort +
                                                             "," + K_DAS_IS_SECURE + "=" + String.valueOf(dasIsSecure) +
                                                             "," + K_DAS_PROTOCOL + "=" + DASProtocol);
            fos.close();
        } catch (IOException ex) {
            throw new CommandException(strings.get("UpdateDasProps.cantWriteProperties", dasPropsFile.getName()), ex);
        }
    }

    /**
     * Return the default value for nodeDirRoot, first checking if com.sun.aas.agentRoot
     * was specified in asenv.conf and returning this value. If not specified,
     * then the defaut value is the {GlassFish_Install_Root}/nodes.
     * nodeDirRoot is the parent directory of the node(s).
     *
     * @return String default nodeDirRoot - parent directory of node(s)
     * @throws CommandException if the GlassFish install root is not found
     */
    private String getNodeDirRootDefault() throws CommandException {
        String nodeDirDefault = getSystemProperty(
                SystemPropertyConstants.AGENT_ROOT_PROPERTY);

        if(StringUtils.ok(nodeDirDefault))
            return nodeDirDefault;

        String installRootPath = getInstallRootPath();
        return installRootPath + "/" + "nodes";
    }

}
